{ hostName, config, lib, pkgs, ... }:

let
  tunnels = lib.filterAttrs (_: wireguard:
    wireguard != null
  ) config.site.hosts.${hostName}.wireguard;
  firstTunnel =
    if builtins.length (builtins.attrNames tunnels) > 0
    then builtins.head (builtins.attrNames tunnels)
    else null;
  enabled = firstTunnel != null;
  privateKeyFile = ifName:
    "/run/wireguard-keys/${ifName}.key";
in
{
  systemd.services = builtins.foldl' (services: ifName: services // {
    "wireguard-key-${ifName}" = {
      description = "Create key file for wireguard interface '${ifName}'";
      requiredBy = [ "systemd-networkd.service" ];
      script = ''
        #! ${pkgs.runtimeShell} -e

        F=${privateKeyFile ifName}
        mkdir -p -m 0700 $(dirname $F)
        chown systemd-network:systemd-network $(dirname $F)
        rm -f $F
        cat >$F <<EOF
        ${tunnels.${ifName}.privateKey}
        EOF
        chmod 0400 $F
        chown systemd-network:systemd-network $F
      '';
    };
  }) {} (builtins.attrNames tunnels);

  environment.systemPackages = lib.optionals enabled [
    pkgs.wireguard-tools
  ];

  systemd.network.netdevs = builtins.mapAttrs (ifName: wireguard: {
    netdevConfig = {
      Name = ifName;
      Kind = "wireguard";
    };
    wireguardConfig.PrivateKeyFile = privateKeyFile ifName;
    wireguardPeers = [ {
      wireguardPeerConfig = {
        PublicKey = wireguard.publicKey;
        Endpoint = wireguard.endpoint;
        AllowedIPs = "0.0.0.0/0, ::/0";
      };
    } ];
  }) tunnels;
  # TODO: qdisc from upstream pillar

  systemd.network.networks = builtins.mapAttrs (ifName: wireguard: {
    matchConfig.Name = ifName;
    addresses = map (addr: {
      addressConfig.Address = addr;
    }) wireguard.addresses;

    networkConfig.DefaultRouteOnDevice = true;

    extraConfig = ''
      [CAKE]
      Parent = root
      # DOCSIS overhead
      OverheadBytes = 18
      Bandwidth = ${toString wireguard.upBandwidth}K
    '';
  }) tunnels;

  networking.nat = lib.optionalAttrs (firstTunnel != null) {
    enable = true;
    externalInterface = firstTunnel;
    forwardPorts = config.site.hosts.${hostName}.forwardedPorts;
  };
}
